/* ------------------------------------------------------------------------------------------------ */
/* FILE :		SysCall_IAR.c																		*/
/*																									*/
/* CONTENTS :																						*/
/*				System call layer extra for IAR CLIB												*/
/*				Provide CLIB with all its native I/O interface										*/
/*																									*/
/*																									*/
/* Copyright (c) 2016-2017, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.5 $																				*/
/*	$Date: 2017/04/23 16:13:16 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "SysCall.h"
#include <yfuns.h>

/* ------------------------------------------------------------------------------------------------ */

extern int IARflush(int);

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __open																				*/
/*																									*/
/* __open - open a file																				*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int __open(const char * filename, int mode);												*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

int __open(const char * filename, int mode)
{
int flags;

	flags = 0;
	if ((mode & _LLIO_CREAT) != 0) {
		flags |= O_CREAT;
	} 
	if ((mode & _LLIO_TRUNC) != 0) {
		flags |= O_TRUNC;
	} 

	mode &= _LLIO_RDWRMASK;

	if (mode == _LLIO_RDONLY) {
		flags |= O_RDONLY;
	}
	else if (mode == _LLIO_WRONLY) {
		flags |= O_WRONLY;
	}
	else {
		flags |= O_RDWR;
	}

	return(open(filename, flags, 0777));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __close																				*/
/*																									*/
/* __close - close a file																			*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int __close(int handle);																	*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

int __close(int handle)
{
	return(close(handle));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __read																				*/
/*																									*/
/* __read - read from  a file																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		size_t __read(int handle, unsigned char * buffer, size_t size);								*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

size_t __read(int handle, unsigned char * buffer, size_t size)
{
	if (handle < 3) {
		size = 1;
	}
	return(read(handle, (void *)buffer, size));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __write																				*/
/*																									*/
/* __write - write to a file																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		size_t __write(int handle, const unsigned char * buffer, size_t size);						*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

size_t __write(int handle, const unsigned char * buffer, size_t size)
{
int Ret;

	Ret = 0;
	if ((buffer == (const unsigned char *)NULL)
	||  (handle < 0)) {
		Ret = IARflush(handle);
	}
	else {
		Ret = write(handle, (const void *)buffer, size);
		if (handle < 3) {
			Ret = size;
		}
	}
	return(Ret);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __lseek																				*/
/*																									*/
/* __lseek - change the location of the read/write													*/
/*																									*/
/* SYNOPSIS:																						*/
/*		long __lseek(int handle, long offset, int whence);											*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

long __lseek(int handle, long offset, int whence)
{
	return((long)lseek(handle, (off_t)offset, whence));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    remove																				*/
/*																									*/
/* remove - delete a file																			*/
/*																									*/
/* SYNOPSIS:																						*/
/*		int remove(const char * filename);															*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

int remove(const char * filename)
{
	return(unlink(filename));
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    abort																				*/
/*																									*/
/* abort - abort the operation																		*/
/*																									*/
/* SYNOPSIS:																						*/
/*		void abort(void);																			*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

void abort(void)
{
	for (;;) {
		TSKselfSusp();
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    clock																				*/
/*																									*/
/* clock - provide the CPU usage of the task														*/
/*																									*/
/* SYNOPSIS:																						*/
/*		clock_t (clock)(void);																		*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

clock_t (clock)(void)
{
  #if ((OX_PERF_MON) > 0)								/* Only available when perf monitoring ON	*/
	return((clock_t)(TSKmyID()->PMtotalRun/(OS_PERF_MON)));
  #elif ((OX_PERF_MON) < 0)
	return((clock_t)(TSKmyID()->PMtotalRun/(-(OX_PERF_MON))));
  #else
	return((clock_t) -1);								/* Not available							*/
  #endif
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __time32																			*/
/*																									*/
/* __time32 - local date / time																		*/
/* SYNOPSIS:																						*/
/*		__time32_t __time32(__time32_t *t);															*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */

__time32_t __time32(__time32_t *t)
{
time_t MyTime;

	MyTime = time(NULL);
	if (*t != NULL) {
		*t = (__time32_t)MyTime;
	}
	return((__time32_t)MyTime);
}

/* ------------------------------------------------------------------------------------------------ */
/* FUNCTION:    __time64																			*/
/*																									*/
/* __time64 - local date / time																		*/
/* SYNOPSIS:																						*/
/*		__time64_t (__time64)(__time64_t *t);														*/
/*																									*/
/* Refer to the CLIB help or the files in IAR's directory ... arm/src/lib for a description			*/
/* ------------------------------------------------------------------------------------------------ */
#if !defined(__ICCAVR32__)
__time64_t __time64(__time64_t *t)
{
time_t MyTime;

	MyTime = time(NULL);
	if (*t != NULL) {
		*t = (__time64_t)MyTime;
	}
	return((__time64_t)MyTime);
}
#endif
/* EOF */
